<?php

namespace Drupal\migrate_wizard\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\migrate_wizard\MWManageDataService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A controller to interact with origin node entities.
 */
class DatabaseController extends ControllerBase {

  /**
   * The MWManageDataService.
   *
   * @var \Drupal\migrate_wizard\MWManageDataService
   */
  protected $mwManageDataService;

  /**
   * {@inheritdoc}
   */
  public function __construct(MWManageDataService $mw_manage_data_service) {
    $this->mwManageDataService = $mw_manage_data_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new self(
      $container->get('migrate_wizard.migrate_wizard_manage_data_service')
    );
  }

 public function databaseSelector($type = null): \Drupal\Core\StringTranslation\TranslatableMarkup
  {
    $current_mw_database = \Drupal::state()->get('current_mw_database');
    $mw_databases = $this->mwManageDataService->entityTypeManager->getStorage('mw_database')->loadMultiple();
    $current_url = \Drupal::request()->getRequestUri();

    $options = '';
    foreach(array_keys($mw_databases) as $key_database){
      $selected = '';
      if($current_mw_database == $key_database){
        $selected = 'selected';
      }

      $options.='<option '.$selected.' value="'.$key_database.'" href="/">'.$key_database.'</option>';
    }

    return $this->t('<span>Select database</span>
                            <form method="post" action="/admin/migrate-wizard/select-database?destination='.$current_url.'">
                                <select name="database_selected" onchange="this.form.submit()"  class="">'.
                                    $options.'
                                </select>
                            </form>');
  }
  /**
   * {@inheritdoc}
   */
  public function selectDatabase() {
    \Drupal::state()->set('current_mw_database', $_REQUEST["database_selected"]);
    $url = \Drupal::service('path.validator')->getUrlIfValid($_REQUEST["destination"]);
    return $this->redirect($url->getRouteName());
  }
}
