<?php

namespace Drupal\migrate_wizard\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\Entity\ConfigEntityListBuilder;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a listing of mw_database entities.
 *
 * @package Drupal\migrate_wizard
 *
 * @ingroup migrate_wizard
 */
class MWDatabaseListBuilder extends ConfigEntityListBuilder {

  /**
   * {@inheritdoc}
   */
  protected $content_types = [];

  protected $migrate_wizard_settings;

  /**
   * Constructs a new EntityListBuilder object.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type definition.
   * @param \Drupal\Core\Entity\EntityStorageInterface $storage
   *   The entity storage class.
   */
  public function __construct(EntityTypeInterface $entity_type, EntityStorageInterface $storage, ConfigFactoryInterface $configFactory) {
    $this->entityTypeId = $entity_type->id();
    $this->storage = $storage;
    $this->entityType = $entity_type;
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity_type.manager')->getStorage($entity_type->id()),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'mw_database_list';
  }

  /**
   * Builds the header row for the entity listing.
   *
   * @return array
   *   A render array structure of header strings.
   *
   * @see \Drupal\Core\Entity\Controller\EntityListController::render()
   */
  public function buildHeader(): array {
    $header['label'] = $this->t('Database');
    $header['id'] = $this->t('Id');
    $header['key'] = $this->t('Key');
    $header['target'] = $this->t('Target');
    // $header['destiny_type'] = $this->t('Destiny Type');
    return $header + parent::buildHeader();
  }

  /**
   * Builds a row for an entity in the entity listing.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity for which to build the row.
   *
   * @return array
   *   A render array of the table row for displaying the entity.
   *
   * @see \Drupal\Core\Entity\EntityListController::render()
   */
  public function buildRow(EntityInterface $entity): array {

    $row['label'] =  $entity->get('database') ? getenv($entity->get('database')) : $entity->get('database');
    $row['id'] = $entity->id();
    $row['key'] = $entity->get('key');
    $row['target'] = $entity->get('target');

    return $row + parent::buildRow($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity): array {
    $operations = parent::getDefaultOperations($entity);
    $operations['list'] = [
      'title' => $this->t('edit'),
      'weight' => 0,
      'url' => Url::fromRoute('entity.database.database_fields',
        ['id' => $entity->id()]
      ),
    ];

    return $operations;
  }
}
