<?php

declare(strict_types = 1);

namespace Drupal\migrate_wizard\Controller;

use Drupal\Core\Config\Entity\ConfigEntityListBuilder;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Url;

/**
 * Provides a listing of origin node entities.
 *
 * @package Drupal\migrate_wizard
 *
 * @ingroup migrate_wizard
 */
class OriginVocabularyListBuilder extends ConfigEntityListBuilder {

  /**
   * {@inheritdoc}
   */
  protected $vocabularies = [];

  protected $current_mw_database;

    public function __construct(EntityTypeInterface $entity_type, EntityStorageInterface $storage) {
      parent::__construct($entity_type, $storage);
      $this->current_mw_database = \Drupal::state()->get('current_mw_database');
  }

  /**
   * Builds the header row for the entity listing.
   *
   * @return array
   *   A render array structure of header strings.
   *
   * @see \Drupal\Core\Entity\Controller\EntityListController::render()
   */
  public function buildHeader(): array {
    $header['label'] = $this->t('Origin node');
    $header['machine_name'] = $this->t('Machine Name');
    $header['description'] = $this->t('Description');
    $header['database'] = $this->t('Database');
    $header['source_type'] = $this->t('Source Type');
    // $header['destiny_type'] = $this->t('Destiny Type');
    return $header + parent::buildHeader();
  }

  /**
   * Builds a row for an entity in the entity listing.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity for which to build the row.
   *
   * @return array
   *   A render array of the table row for displaying the entity.
   *
   * @see \Drupal\Core\Entity\EntityListController::render()
   */
  public function buildRow(EntityInterface $entity): array {
    $entity_mw_database = $entity->get('use_env') ? getenv($entity->get('database')) : $entity->get('database');
    if($this->current_mw_database == $entity_mw_database){
      $row['label'] = $entity->label();
      $row['machine_name'] = $entity->id();
      $row['description'] = $entity->get('description');
      $row['database'] = $entity_mw_database;
      $row['source_type'] = $entity->get('source_type');
      return $row + parent::buildRow($entity);
    }else{
      return [];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity): array {
    $operations = parent::getDefaultOperations($entity);
    $operations['list'] = [
      'title' => $this->t('Manage Fields Origin Vocabularies'),
      'weight' => 0,
      'url' => Url::fromRoute('entity.origin_vocabularies.origin_vocabularies_fields',
        ['origin_bundle' => $entity->id()]
      ),
    ];

    return $operations;
  }
}
