<?php

namespace Drupal\migrate_wizard;

use Drupal\Component\Plugin\Factory\DefaultFactory;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\migrate\Plugin\MigrationInterface;

/**
 * Manager class for the platform plugins.
 */
class FieldTypeMWManager extends DefaultPluginManager implements FieldTypeMWManagerInterface{

  /**
   * {@inheritdoc}
   */
  protected $entityFieldManager;

  /**
   * {@inheritdoc}
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  protected $migration;
  /**
   * Constructs an SocialLinkFieldPlatformManager object.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   */
  public function __construct(\Traversable $namespaces,
                              CacheBackendInterface $cache_backend,
                              ModuleHandlerInterface $module_handler,
                              EntityFieldManagerInterface $entity_field_manager,
                              ConfigFactoryInterface $configFactory
                              ) {
    parent::__construct('Plugin/FieldTypeMW', $namespaces, $module_handler, NULL, 'Drupal\migrate_wizard\Annotation\FieldType');
    $this->entityFieldManager = $entity_field_manager;
    $this->config = $configFactory->getEditable('migrate_wizard.settings');
  }

  /**
   * {@inheritdoc}
   */
  public function createInstance($plugin_id, array $configuration = []) {
    $plugin_definition = $this->getDefinition($plugin_id);
    $plugin_class = DefaultFactory::getPluginClass($plugin_id, $plugin_definition);

    // If the plugin provides a factory method, pass the container to it.
    if (is_subclass_of($plugin_class, 'Drupal\Core\Plugin\ContainerFactoryPluginInterface')) {
      return $plugin_class::create(\Drupal::getContainer(), $configuration, $plugin_id, $plugin_definition);
    }

    return new $plugin_class($configuration, $plugin_id, $plugin_definition, $this->entityFieldManager, $this->config);
  }


  /**
   * {@inheritdoc}
   */
  public function createCustomInstance($plugin_id, $migration, $state, array $configuration = []) {
    $plugin_definition = $this->getDefinition($plugin_id);
    $plugin_class = DefaultFactory::getPluginClass($plugin_id, $plugin_definition);

    // If the plugin provides a factory method, pass the container to it.
//    if (is_subclass_of($plugin_class, 'Drupal\Core\Plugin\ContainerFactoryPluginInterface')) {
//      return $plugin_class::create(\Drupal::getContainer(), $configuration, $plugin_id, $plugin_definition, $migration);
//    }

    return new $plugin_class($configuration, $plugin_id, $plugin_definition, $migration, $state);
  }
  /**
   * Get all platform plugins.
   *
   * @return array
   *   The platform plugins.
   */
  public function getFieldTypes() {
    return $this->getDefinitions();
  }

  public function createReflectionClass($plugin_id) {

    $plugin_definition = $this->getDefinition($plugin_id);
    $plugin_class = DefaultFactory::getPluginClass($plugin_id, $plugin_definition);

    return new \ReflectionClass($plugin_class);
  }

}
