<?php

namespace Drupal\migrate_wizard\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ConfigManager;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ExtensionList;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Serialization\Yaml;
use Drupal\migrate_wizard\MWManageDataService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Database\Database;

/**
 * Configure migrate_wizard conection settings for this site.
 */
class MigrateWizardSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  protected $extensionListModule;

  /**
   * {@inheritdoc}
   */
  protected $configManager;

  /**
   * {@inheritdoc}
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  protected $mwManageDataService;

  /**
   * {@inheritdoc}
   */
  public function __construct(ConfigFactoryInterface $configFactory, ExtensionList $extension_list_module, ConfigManager $config_manager, EntityTypeManagerInterface $entity_type_manager, MWManageDataService $mw_manage_data_service) {
    $this->config = $configFactory->getEditable('migrate_wizard.settings');
    $this->extensionListModule = $extension_list_module;
    $this->configManager = $config_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->mwManageDataService = $mw_manage_data_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new self(
      $container->get('config.factory'),
      $container->get('extension.list.module'),
      $container->get('config.manager'),
      $container->get('entity_type.manager'),
      $container->get('migrate_wizard.migrate_wizard_manage_data_service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'migrate_wizard';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['migrate_wizard_settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

//     $key = 'migrate2';
//     $target = 'ceagu';
//     $info = [
//        'host' => getenv('MYSQL_HOST'),
//        'database' => 'ceagu',
//        'username' => getenv('MYSQL_USER'),
//        'password' => getenv('MYSQL_PASSWORD'),
//        'prefix' => getenv('DB_PREFIX'),
//        'driver' => getenv('DB_DRIVER'),
//        'port' => 3306,
//        'namespace' => 'Drupal\\Core\\Database\\Driver\\mysql',
//        'collation' => 'utf8mb4_general_ci'
//     ];
//    Database::addConnectionInfo($key, $target,  $info);
//
//    Database::setActiveConnection($key);
    $current_config = $this->config->get('migrate_wizard_settings');

    $form['migration_group_name'] = [
      '#type' => 'textfield',
      '#cols' => 20,
      '#title' => $this->t('Migration Group Name'),
      '#default_value' => $current_config['migration_group_name'] ?? '',
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {}

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    parent::submitForm($form, $form_state);
    $user_inputs = $form_state->getUserInput();
    $this->config->set('migrate_wizard_settings', $user_inputs)->save();

    if ($user_inputs["migration_group_name"]) {
      $module_path = $this->extensionListModule->getPath('migrate_wizard');
      $migration_group_widlcard_config = Yaml::decode(file_get_contents($module_path . '/config_widlcards/migrate_plus.migration_group.widlcard.yml'));
      if (!is_array($migration_group_widlcard_config)) {
        throw new \RuntimeException('Invalid YAML file', 'file');
      }
      $migration_group_widlcard_config['id'] = $user_inputs["migration_group_name"];
      $migration_group_widlcard_config['label'] = $user_inputs["migration_group_name"];

      $migration_group_widlcard_config['langcode'] = $this->mwManageDataService->getOriginDefaultLangcode($this->config->get('migrate_wizard_settings'));

      $entity_config = $this->configManager->loadConfigEntityByName("migrate_plus.migration_group.{$migration_group_widlcard_config['id']}");
      $type = $this->configManager->getEntityTypeIdByName(basename('migrate_plus.migration_group.wildcard.yml'));
      $entity_storage = $this->entityTypeManager->getStorage($type);

      if ($entity_config) {
        $entity = $entity_storage->updateFromStorageRecord($entity_config, $migration_group_widlcard_config);
        $entity->save();
      }
      else {
        $entity = $entity_storage->createFromStorageRecord($migration_group_widlcard_config);
        $entity->save();
      }
    }
  }
  public function getTitle($type = null): \Drupal\Core\StringTranslation\TranslatableMarkup
  {
    return $this->t('Migrate wizard settings');
  }
}
