<?php

namespace Drupal\migrate_wizard\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\migrate_wizard\MWConfigBuilder;
use Drupal\migrate_wizard\MWManageDataService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure migrate_wizard.
 */
class OriginContentTypeFields extends BaseFormFields {

  use BaseFormFieldsTrait;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'manage_origin_content_type_fields';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $origin_bundle = NULL) {
    $this->originType = 'node';
    $origin_node_entity = $this->entityTypeManager->getStorage('origin_node')->load($this->originBundle);
    $this->currentConfig = $origin_node_entity->shared_configuration;
    $this->manageFormService->setCurrentConfig($origin_node_entity->shared_configuration);
    $this->fieldsOriginBundle = $this->manageFormService->getFieldsOriginBundle($origin_node_entity, $this->originType);
    $this->manageFormService->setFieldsOriginBundle($this->fieldsOriginBundle);
    $type_fields_checkboxes = [
      'preserve_nid' => 'Preserve nid',
      'import_users' => 'Import users',
      'import_path_alias' => 'Import path alias',
    ];
    $this->checkboxes = array_merge($this->checkboxes, $type_fields_checkboxes);
    $form['origin_type'] = [
      '#type' => 'hidden',
      '#value' => 'node',
    ];

    $form['origin_bundle'] = [
      '#type' => 'hidden',
      '#value' => $origin_node_entity->id_origin,
    ];

    $origin_database_value = $origin_node_entity->use_env ? getenv($origin_node_entity->database) : $origin_node_entity->database;
    $form['origin_database'] = [
      '#type' => 'hidden',
      '#value' => $origin_database_value,
    ];
    $this->langCodes['default_lang']['value'] = $this->mwManageDataService->getOriginDefaultLangcode();

    $config_translation = $this->mwManageDataService->getOriginLanguages($form["origin_type"]["#value"], $origin_bundle);

    foreach($config_translation["origin_languages"] as $language){
      $this->langCodes[$language.'_lang'] = [
        'markup' => $language,
        'value' => $language,
      ];
    }

    return parent::buildForm($form, $form_state, $origin_node_entity);
  }
  public function getTitle($type = null): \Drupal\Core\StringTranslation\TranslatableMarkup
  {
    return parent::getTitle('node');
  }
}
