<?php

namespace Drupal\migrate_wizard\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\migrate_wizard\MWConfigBuilder;
use Drupal\migrate_wizard\MWManageDataService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure migrate_wizard.
 */
class OriginUserFields extends BaseFormFields {

  use BaseFormFieldsTrait;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'manage_origin_user_fields';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $origin_bundle = NULL) {
    $this->originType = 'user';
    $this->destinyType = 'user';

    if(isset($this->mwManageDataService->mw_database->shared_configuration['user_migration_config'])){
      $this->currentConfig = $this->mwManageDataService->mw_database->shared_configuration['user_migration_config'];
      $this->manageFormService->setCurrentConfig($this->mwManageDataService->mw_database->shared_configuration['user_migration_config']);
    }

    $origin_entity = new \stdClass();
    $origin_entity->id_origin =  'user';
    $origin_entity->key =  $this->mwManageDataService->mw_database->key;
    $origin_entity->target =  $this->mwManageDataService->mw_database->target;

    $origin_database_value = $this->mwManageDataService->mw_database->use_env ? getenv($this->mwManageDataService->mw_database->database) : $this->mwManageDataService->mw_database->database;
    $form['origin_database'] = [
      '#type' => 'hidden',
      '#value' => $origin_database_value,
    ];

    $this->fieldsOriginBundle = $this->manageFormService->getFieldsOriginBundle($origin_entity, $this->originType);
    $this->manageFormService->setFieldsOriginBundle($this->fieldsOriginBundle);
    $type_fields_checkboxes = [
      'preserve_uid' => 'Preserve uid',
      'import_path_alias' => 'Import path alias',
    ];

    $moduleHandler = \Drupal::service('module_handler');
    if($this->mwManageDataService->checkIfCasModuleOriginExistsEnabled() && $moduleHandler->moduleExists('media')){
      $type_fields_checkboxes['import_cas'] = 'Import cas';
    }

    $this->checkboxes = array_merge($this->checkboxes, $type_fields_checkboxes);
    $form['origin_type'] = [
      '#type' => 'hidden',
      '#value' => 'user',
    ];

    $this->langCodes['default_lang']['value'] = $this->mwManageDataService->getOriginDefaultLangcode();

    $config_translation = $this->mwManageDataService->getOriginLanguages($form["origin_type"]["#value"], $origin_bundle);

    foreach($config_translation["origin_languages"] as $language){
      $this->langCodes[$language.'_lang'] = [
        'markup' => $language,
        'value' => $language,
      ];
    }

    return parent::buildForm($form, $form_state, $origin_entity);
  }
  public function getTitle($type = null): \Drupal\Core\StringTranslation\TranslatableMarkup
  {
    return $this->t('Manage fields User');
  }
}
