<?php

namespace Drupal\migrate_wizard;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Component\Transliteration\PhpTransliteration;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ConfigManager;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ExtensionList;
use Drupal\Core\Serialization\Yaml;

/**
 * Defines a Service to update nodes.
 */
class MWConfigBuilder {

  /**
   * {@inheritdoc}
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  protected $entityFieldManager;

  /**
   * {@inheritdoc}
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  protected $migrationUrlFilesProd;

  /**
   * {@inheritdoc}
   */
  protected $extensionListModule;

  /**
   * {@inheritdoc}
   */
  protected $configManager;

  /**
   * {@inheritdoc}
   */
  protected $transliterator;

  /**
   * {@inheritdoc}
   */
  protected $fieldTypeMWManager;


  /**
   * {@inheritdoc}
   */
  protected $migration;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityFieldManagerInterface $entity_field_manager,
                              EntityTypeManagerInterface $entity_type_manager,
                              ConfigFactoryInterface $config_factory,
                              ExtensionList $extension_list_module,
                              ConfigManager $config_manager,
                              PhpTransliteration $php_transliteration,
                              FieldTypeMWManagerInterface $field_type_mw_manager) {
    $this->config = $config_factory->getEditable('migrate_wizard.settings');
    $this->configFactory = $config_factory;
//    if (is_array($this->config->get('migrate_wizard_settings'))) {
//      $this->migrationUrlFilesProd = $this->config->get('migrate_wizard_settings')['migration_url_files_prod'];
//    }
    $this->entityFieldManager = $entity_field_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->extensionListModule = $extension_list_module;
    $this->configManager = $config_manager;
    $this->transliterator = $php_transliteration;
    $this->fieldTypeMWManager = $field_type_mw_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function buildMigrationConfig($origin_node, $array_config) {
    unset($array_config['build_migration_config']);
    $this->buildConfigByLanguage($origin_node, $array_config, $array_config[array_key_first($array_config)]["languages"]["default"]);
    foreach($array_config[array_key_first($array_config)]["languages"] as $key => $language){
      if($key != 'default'){
        $this->buildConfigByLanguage($origin_node, $array_config, $array_config[array_key_first($array_config)]["languages"]["default"], $array_config[array_key_first($array_config)]["languages"][$language], TRUE);
      }
    }
  }

  private function buildConfigByLanguage($origin_node, $array_config, $default_langcode = NULL, $langcode = NULL, $translation = FALSE){
    $module_path = $this->extensionListModule->getPath('migrate_wizard');

    $yml_type = $array_config["origin_type"];
    if($array_config["origin_type"] == 'field_collection'){
      $yml_type = 'field_collection';
    }

    $mw_database = $this->entityTypeManager->getStorage('mw_database')->load(\Drupal::state()->get('current_mw_database'));
    $shared_configuration = $mw_database->get('shared_configuration');

    $wildcard_yml_config = Yaml::decode(file_get_contents($module_path . "/config_widlcards/migrate_plus.migration.wildcard_{$yml_type}.yml"));
    $field_type_mw = $this->preparePlugin('default_'.$yml_type);
    $field_type_mw->name::getD7MigrationConfig($wildcard_yml_config, $array_config, $origin_node, $this->config->get('migrate_wizard_settings')['migration_group_name'], $default_langcode, $langcode, $translation);
    foreach ($array_config[array_key_first($array_config)] as $origin_field => $data_field) {
      if ((isset($data_field['destiny']) && $data_field['destiny'] != 'none') && $origin_field != 'languages') {
        $field_type_mw = $this->preparePlugin($data_field['type']);
        $field_type_mw->name::getD7MigrationConfig($wildcard_yml_config, $origin_field, $data_field, $array_config["destiny_type"], $shared_configuration, $this->entityFieldManager, $this->fieldTypeMWManager, $langcode);
      }
    }
    if($translation){
      $wildcard_yml_config['migration_dependencies']['required'] = [$origin_node];
    }
    $this->installConfig($wildcard_yml_config);
  }

  /**
   * {@inheritdoc}
   */
  public function buildMigrationUserConfig($origin_node, $array_config) {

    unset($array_config['build_migration_config']);
    $module_path = $this->extensionListModule->getPath('migrate_wizard');
    $wildcard_yml_config = Yaml::decode(file_get_contents($module_path . '/config_widlcards/migrate_plus.migration.wildcard_users.yml'));

    $field_type_mw = $this->preparePlugin('default_user');
    $field_type_mw->name::getD7MigrationConfig($wildcard_yml_config, $array_config, $origin_node, $this->config->get('migrate_wizard_settings')['migration_group_name'], $this->entityTypeManager->getStorage('mw_database')->load(\Drupal::state()->get('current_mw_database'))->get('shared_configuration')['user_roles']);

    foreach ($array_config[array_key_first($array_config)] as $origin_field => $data_field) {
      if ((isset($data_field['destiny']) && $data_field['destiny'] != 'none') && $origin_field != 'languages') {
        $field_type_mw = $this->preparePlugin($data_field['type']);
        $field_type_mw->name::getD7MigrationConfig($wildcard_yml_config, $origin_field, $data_field, $array_config["destiny_type"], $this->config, $this->entityFieldManager, $this->fieldTypeMWManager);
      }
    }

    $this->installConfig($wildcard_yml_config);
  }

  /**
   * {@inheritdoc}
   */
  private function installConfig($wildcard_yml_config) {

    if (!is_array($wildcard_yml_config)) {
      throw new \RuntimeException('Invalid YAML file', 'file');
    }

    $entity_config = $this->configManager->loadConfigEntityByName("migrate_plus.migration.{$wildcard_yml_config['id']}");
    $entity_storage = $this->entityTypeManager->getStorage('migration');

    if ($entity_config) {
      $entity = $entity_storage->updateFromStorageRecord($entity_config, $wildcard_yml_config);
      $entity->save();
    }
    else {
      $entity = $entity_storage->createFromStorageRecord($wildcard_yml_config);
      $entity->save();
    }
  }

  /**
   * Prepares the tamper plugin.
   *
   * @param string $field_type_mw_id
   *   The id of the field type plugin.
   *
   * @return FieldTypeMWInterface|null
   *   The tamper plugin instance or null in case the Tamper plugin could not be
   *   instantiated.
   * @throws PluginException
   */
  protected function preparePlugin($field_type_mw_id) {
    return $this->fieldTypeMWManager->createReflectionClass($field_type_mw_id);
  }
}
