<?php

namespace Drupal\migrate_wizard\Plugin\migrate\process;

use Drupal\link\Plugin\migrate\process\FieldLink;
use Drupal\migrate\MigrateExecutableInterface;
use Drupal\migrate\Row;

/**
 * Transform a pre-Drupal 8 formatted link for use in Drupal 8.
 *
 * Previous to Drupal 8, URLs didn't need to have a URI scheme assigned. The
 * contrib link module would auto-prefix the URL with a URI scheme. A link in
 * Drupal 8 has more validation and external links must include the URI scheme.
 * All external URIs need to be converted to use a URI scheme.
 *
 * Available configuration keys
 * - uri_scheme: (optional) The URI scheme prefix to use for URLs without a
 *   scheme. Defaults to 'http://', which was the default in Drupal 6 and
 *   Drupal 7.
 *
 * Examples:
 *
 * Consider a link field migration, where you want to use https:// as the
 * prefix:
 *
 * @code
 * process:
 *   field_link:
 *     plugin: field_link_value
 *     uri_scheme: 'https://'
 *     source: field_link
 * @endcode
 *
 * @MigrateProcessPlugin(
 *   id = "text_long_with_format"
 * )
 */
class TextLongWithFormat extends FieldLink {

  /**
   * {@inheritdoc}
   */
  public function transform($value, MigrateExecutableInterface $migrate_executable, Row $row, $destination_property) {

    $array_values = [];
    if($value[0] != NULL){
      foreach ($value[0] as $key => $val) {
        $array_values[] = [
          'value' => $val,
          'format' => $value[1][$key],
        ];
      }
    }
    return $array_values;
  }

}
