<?php

namespace Drupal\migrate_wizard\Plugin\migrate\source\d7;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\DatabaseExceptionWrapper;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\migrate\Row;
use Drupal\migrate_wizard\FieldTypeMWManagerInterface;
use Drupal\paragraphs\Plugin\migrate\source\d7\ParagraphsItem;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Paragraphs Item source plugin.
 *
 * Available configuration keys:
 * - bundle: (optional) If supplied, this will only return paragraphs
 *   of that particular type.
 *
 * @MigrateSource(
 *   id = "mw_d7_paragraphs_item",
 *   source_module = "paragraphs",
 * )
 */
class MWD7ParagraphsItem extends ParagraphsItem {

   /**
   * The config of content type to import.
   *
   * @var string
   */
  protected $config;

  /**
   * The config of content type to import.
   *
   * @var string
   */
  protected $configFactory;

  /**
   * Array of bundles.
   *
   * @var array
   */
  public $bundle;

  /**
   * Array of nid or vid destiny type.
   *
   * @var string
   */
  protected $destinyType;

  /**
   * Array of machine name of origin to import.
   *
   * @var string
   */
  protected $originType;

  /**
   * Query Sql to add conditions.
   *
   * @var Query
   */
  public $query;

  /**
   * The left join table.
   *
   * @var string
   */
  protected $leftJoinTable;

  /**
   * The current config of content to import.
   *
   * @var array
   */
  protected $currentConfig;


  /**
   * {@inheritdoc}
   */
  protected $fieldTypeMWManager;

  /**
   * {@inheritdoc}
   */
  protected $leftTable;

  /**
   * {@inheritdoc}
   */
  protected $leftTableId;

  /**
   * {@inheritdoc}
   */
  protected $entityType;

  /**
   * {@inheritdoc}
   */
  protected $translation;
  /**
   * {@inheritdoc}
   */
  protected $defaultLangcode;
    /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration,
                              $plugin_id,
                              $plugin_definition,
                              MigrationInterface $migration,
                              StateInterface $state,
                              EntityTypeManagerInterface $entity_type_manager,
                              ConfigFactoryInterface $config_factory,
                              FieldTypeMWManagerInterface $field_type_mw_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $migration, $state, $entity_type_manager);


    $this->state = $state;
    $current_source_config = $migration->getSourceConfiguration();
    $this->config = $config_factory->getEditable('migrate_wizard.settings');

    $this->bundle = $current_source_config["field_name"];
//    if(isset($current_source_config["constants"]["translations"]) && $current_source_config["constants"]["translations"]){
//      $this->bundle = str_replace('_'.$current_source_config["constants"]["langcode"], '', $current_source_config['bundle']);
//    }
    $this->currentConfig = $this->config->get($this->bundle);

    $this->defaultLangcode = $configuration["constants"]["default_langcode"];

    $this->translation = ['und'];
    if(isset($this->currentConfig[array_key_first($this->currentConfig)]["languages"])){
      $this->translation = ['und', $this->currentConfig[array_key_first($this->currentConfig)]["languages"]["default"]];
    }

    if(isset($configuration["constants"]["translations"]) && $configuration["constants"]["translations"]){
      $this->translation = [$configuration["constants"]["langcode"]];
    }
    $this->configFactory = $config_factory;
    $this->fieldTypeMWManager = $field_type_mw_manager;
  }

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration = NULL) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $migration,
      $container->get('state'),
      $container->get('entity_type.manager'),
      $container->get('config.factory'),
      $container->get('plugin.manager.field_type_mw')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    $query = parent::query();

    $a = $query->execute()->fetchAll();
    return $query;
  }

    /**
   * {@inheritdoc}
   */
  public function prepareRow(Row $row) {

    $prepare_row_result = parent::prepareRow($row);

    if($row->hasSourceProperty('parent_type')){
      $lang = $this->getLangcode($row->getSourceProperty('parent_type'), $row->getSourceProperty('field_name'), $row->getSourceProperty('item_id'));

      $row->setSourceProperty('language', $lang);
      $row->setSourceProperty('langcode', $lang);
    }


    [
      'item_id' => $paragraph_id,
      'revision_id' => $paragraph_revision_id,
      'field_name' => $paragraph_parent_field_name,
      'bundle' => $bundle,
    ] = $row->getSource();

    // Get field API field values.
    foreach (array_keys($this->getFields('paragraphs_item', $bundle)) as $field) {
      if(isset(reset($this->currentConfig)[$field]) && reset($this->currentConfig)[$field]['type'] == 'paragraphs'){
        $new_field_source = [];
        $field_source = $row->getSourceProperty($field);
        foreach($field_source as $field_source_key => $field_source_value){
          $new_field_source[$field_source_key] = ['item_id' => $field_source_value['value'], 'revision_id' => $field_source_value['revision_id']];
        }
        $row->setSourceProperty($field, $new_field_source);
      }
    }

    foreach ($this->currentConfig[array_key_first($this->currentConfig)] as $field_name => $data_field) {
      if(isset($data_field['type']) && $data_field["destiny"] != 'none'){
        switch ($data_field['type']) {
          case 'image':
            $new_value = [];
            foreach($row->getSourceProperty($field_name) as $key => $value){
              $query_uri = $this->select('file_managed', 'fm');
              $query_uri->fields('fm', ['uri']);
              $query_uri->leftJoin('field_data_field_file_image_alt_text', "ffat", "fm.fid = ffat.entity_id");
              $query_uri->fields('ffat', ['field_file_image_alt_text_value']);
              $query_uri->leftJoin('field_data_field_file_image_title_text', "fftt", "fm.fid = fftt.entity_id");
              $query_uri->fields('fftt', ['field_file_image_title_text_value']);
              $query_uri->condition('fm.fid', $value['fid']);
              $results = $query_uri->execute()->fetch();
              $new_value[$key] = $value;
              if ($results["field_file_image_alt_text_value"]) {
                $new_value[0]["alt"] = $results["field_file_image_alt_text_value"];
              }
              if ($results["field_file_image_title_text_value"]) {
                $new_value[0]["title"] = $results["field_file_image_title_text_value"];
              }
              $new_value[$key]['uri'] = $results['uri'];
              $new_value[$key]['type'] = $data_field['type'];
            }
            $row->setSourceProperty($field_name, $new_value);
            break;
          case 'file':
            $new_value = [];
            foreach($row->getSourceProperty($field_name) as $key => $value){
              $query_uri = $this->select('file_managed', 'fm');
              $query_uri->fields('fm', ['uri']);
              $query_uri->condition('fm.fid', $value['fid']);
              $uri = $query_uri->execute()->fetchField();
              $new_value[$key] = $value;
              $new_value[$key]['uri'] = $uri;
              $new_value[$key]['type'] = $data_field['type'];
            }
            $row->setSourceProperty($field_name, $new_value);
            break;
          case 'datetime':
            $date = $row->getSourceProperty($field_name . '_value');
            if (isset($data_field['format']) && $data_field['format'] != '' && $date != '') {
              $date = DrupalDateTime::createFromFormat('Y-m-d H:i:s', $date);
              $date->format($data_field['format']);
              $row->setSourceProperty($field_name . '_value', $date->format($data_field['format']));

            }
            break;
//          case 'list_text':
//            if ($data_field['destiny_type'] == 'entity_reference') {
//              $this->listTextToTaxonomy($row, $field_name, $data_field);
//            }
//            break;
        }
      }
    }

    return $prepare_row_result;
  }

    /**
   * {@inheritdoc}
   */
  public function getIds() {
    $ids = [
      'item_id' => [
        'type' => 'integer',
        'alias' => 'p',
      ],
      'revision_id' => [
        'type' => 'integer',
        'alias' => 'p',
      ]
    ];

    return $ids;
  }

  private function getLangcode($parent_type, $field_name, $item_id, $depth = 0){
    if($parent_type == 'paragraphs_item' || $depth == 0){
      $query = $this->select('field_data_'.$field_name, 'fdc')
      ->fields('fdc', ['entity_type', 'bundle', 'entity_id']);
      $query->condition('fdc.'.$field_name.'_value', $item_id);
      $query->leftJoin('paragraphs_item', 'pi', 'fdc.'.'entity_id = pi.item_id');
      $query->addField('pi', 'field_name');
      $result = $query->execute()->fetchAssoc();
      return $this->getLangcode($result['entity_type'], $result['field_name'], $result['entity_id'], $depth + 1);
    }else{

      $query = $this->select('node', 'n');
      $query->condition('nid', $item_id);
      $query->addField('n', 'language', 'language');

      return $query->execute()->fetchField();
    }
  }
}
